@@ -6,10 +6,9 @@
 # This work is licensed under the GNU GPLv2 or later.
 # See the COPYING file in the top-level directory.
 
-import logging
 import pwd
 
-from .domain import DomainCpu
+from .logger import log
 from .xmlbuilder import XMLBuilder, XMLChildProperty, XMLProperty
 
 
@@ -17,27 +16,11 @@ from .xmlbuilder import XMLBuilder, XMLChildProperty, XMLProperty
 # capabilities host <cpu> parsing #
 ###################################
 
-class _CapsCPU(DomainCpu):
-    arch = XMLProperty("./arch")
-
-
-###########################
-# Caps <topology> parsers #
-###########################
-
-class _CapsTopologyCPU(XMLBuilder):
+class _CapsCPU(XMLBuilder):
     XML_NAME = "cpu"
-    id = XMLProperty("./@id")
-
-
-class _TopologyCell(XMLBuilder):
-    XML_NAME = "cell"
-    cpus = XMLChildProperty(_CapsTopologyCPU, relative_xpath="./cpus")
-
-
-class _CapsTopology(XMLBuilder):
-    XML_NAME = "topology"
-    cells = XMLChildProperty(_TopologyCell, relative_xpath="./cells")
+    arch = XMLProperty("./arch")
+    model = XMLProperty("./model")
+    vendor = XMLProperty("./vendor")
 
 
 ######################################
@@ -60,7 +43,6 @@ class _CapsHost(XMLBuilder):
     XML_NAME = "host"
     secmodels = XMLChildProperty(_CapsSecmodel)
     cpu = XMLChildProperty(_CapsCPU, is_single=True)
-    topology = XMLChildProperty(_CapsTopology, is_single=True)
 
     def get_qemu_baselabel(self):
         for secmodel in self.secmodels:
@@ -73,7 +55,7 @@ class _CapsHost(XMLBuilder):
                     label = baselabel.content
                     break
             if not label:
-                continue
+                continue  # pragma: no cover
 
             # XML we are looking at is like:
             #
@@ -88,7 +70,7 @@ class _CapsHost(XMLBuilder):
                 user = pwd.getpwuid(uid)[0]
                 return user, uid
             except Exception:
-                logging.debug("Exception parsing qemu dac baselabel=%s",
+                log.debug("Exception parsing qemu dac baselabel=%s",
                     label, exc_info=True)
         return None, None
 
@@ -116,6 +98,7 @@ class _CapsGuestFeatures(XMLBuilder):
     pae = XMLProperty("./pae", is_bool=True)
     acpi = XMLProperty("./acpi/@default", is_onoff=True)
     apic = XMLProperty("./apic/@default", is_onoff=True)
+    externalSnapshot = XMLProperty("./externalSnapshot", is_bool=True)
 
 
 class _CapsGuest(XMLBuilder):
@@ -138,33 +121,33 @@ class _CapsGuest(XMLBuilder):
     def all_machine_names(self, domain):
         """
         Return all machine string names, including canonical aliases for
-        the guest+domain combo
+        the guest+domain combo but avoiding duplicates
         """
         mobjs = (domain and domain.machines) or self.machines
         ret = []
         for m in mobjs:
             ret.append(m.name)
-            if m.canonical:
-                ret.append(m.canonical)
+            if m.canonical and m.canonical not in ret:
+                ret.append(m.canonical)  # pragma: no cover
         return ret
 
-    def has_install_options(self):
+    def is_machine_alias(self, domain, src, tgt):
         """
-        Return True if there are any install options available
+        Determine if machine @src is an alias for machine @tgt
         """
-        return bool(len(self.domains) > 0)
+        mobjs = (domain and domain.machines) or self.machines
+        for m in mobjs:
+            if m.name == src and m.canonical == tgt:
+                return True
+        return False
 
     def is_kvm_available(self):
         """
         Return True if kvm guests can be installed
         """
-        if self.os_type != "hvm":
-            return False
-
         for d in self.domains:
             if d.hypervisor_type == "kvm":
                 return True
-
         return False
 
     def supports_pae(self):
@@ -185,6 +168,12 @@ class _CapsGuest(XMLBuilder):
         """
         return bool(self.features.apic)
 
+    def supports_externalSnapshot(self):
+        """
+        Return True if capabilities report support for external snapshots
+        """
+        return bool(self.features.externalSnapshot)
+
 
 ############################
 # Main capabilities object #
@@ -208,6 +197,9 @@ class _CapsInfo(object):
         self.emulator = self.domain.emulator or self.guest.emulator
         self.machines = self.guest.all_machine_names(self.domain)
 
+    def is_machine_alias(self, src, tgt):
+        return self.guest.is_machine_alias(self.domain, src, tgt)
+
 
 class Capabilities(XMLBuilder):
     def __init__(self, *args, **kwargs):
@@ -220,55 +212,11 @@ class Capabilities(XMLBuilder):
     guests = XMLChildProperty(_CapsGuest)
 
 
-    ###################
-    # Private helpers #
-    ###################
-
-    def _is_xen(self):
-        for g in self.guests:
-            if g.os_type != "xen":
-                continue
-
-            for d in g.domains:
-                if d.hypervisor_type == "xen":
-                    return True
-
-        return False
-
-
-    ##############
-    # Public API #
-    ##############
-
-    def get_cpu_values(self, arch):
-        if not arch:
-            return []
-        if not self.conn.check_support(self.conn.SUPPORT_CONN_CPU_MODEL_NAMES):
-            return []
-        if arch in self._cpu_models_cache:
-            return self._cpu_models_cache[arch]
-
-        try:
-            names = self.conn.getCPUModelNames(arch, 0)
-            if names == -1:
-                names = []
-        except Exception as e:
-            logging.debug("Error fetching CPU model names for arch=%s: %s",
-                          arch, e)
-            names = []
-
-        self._cpu_models_cache[arch] = names
-        return names
-
-
     ############################
     # Public XML building APIs #
     ############################
 
     def _guestForOSType(self, os_type, arch):
-        if self.host is None:
-            return None
-
         archs = [arch]
         if arch is None:
             archs = [self.host.cpu.arch, None]
@@ -296,7 +244,7 @@ class Capabilities(XMLBuilder):
         if not domains:
             return None
 
-        priority = ["kvm", "xen", "qemu"]
+        priority = ["kvm", "xen", "hvf", "qemu"]
 
         for t in priority:
             for d in domains:
@@ -306,6 +254,15 @@ class Capabilities(XMLBuilder):
         # Fallback, just return last item in list
         return domains[-1]
 
+    def has_install_options(self):
+        """
+        Return True if there are any install options available
+        """
+        for guest in self.guests:
+            if guest.domains:
+                return True
+        return False
+
     def guest_lookup(self, os_type=None, arch=None, typ=None, machine=None):
         """
         Simple virtualization availability lookup
@@ -337,27 +294,37 @@ class Capabilities(XMLBuilder):
 
         guest = self._guestForOSType(os_type, arch)
         if not guest:
-            archstr = _("for arch '%s'") % arch
-            if not arch:
-                archstr = ""
-
-            osstr = _("virtualization type '%s'") % os_type
-            if not os_type:
-                osstr = _("any virtualization options")
-
-            raise ValueError(_("Host does not support %(virttype)s %(arch)s") %
-                               {'virttype': osstr, 'arch': archstr})
+            if arch and os_type:
+                msg = (_("Host does not support virtualization type "
+                         "'%(virttype)s' for architecture '%(arch)s'") %
+                         {'virttype': os_type, 'arch': arch})
+            elif arch:
+                msg = (_("Host does not support any virtualization options "
+                         "for architecture '%(arch)s'") %
+                         {'arch': arch})
+            elif os_type:
+                msg = (_("Host does not support virtualization type "
+                         "'%(virttype)s'") %
+                         {'virttype': os_type})
+            else:
+                msg = _("Host does not support any virtualization options")
+            raise ValueError(msg)
 
         domain = self._bestDomainType(guest, typ, machine)
         if domain is None:
-            machinestr = " with machine '%s'" % machine
-            if not machine:
-                machinestr = ""
-            raise ValueError(_("Host does not support domain type %(domain)s"
-                               "%(machine)s for virtualization type "
-                               "'%(virttype)s' arch '%(arch)s'") %
-                               {'domain': typ, 'virttype': guest.os_type,
-                                'arch': guest.arch, 'machine': machinestr})
+            if machine:
+                msg = (_("Host does not support domain type %(domain)s with "
+                         "machine '%(machine)s' for virtualization type "
+                         "'%(virttype)s' with architecture '%(arch)s'") %
+                         {'domain': typ, 'virttype': guest.os_type,
+                         'arch': guest.arch, 'machine': machine})
+            else:
+                msg = (_("Host does not support domain type %(domain)s for "
+                         "virtualization type '%(virttype)s' with "
+                         "architecture '%(arch)s'") %
+                         {'domain': typ, 'virttype': guest.os_type,
+                         'arch': guest.arch})
+            raise ValueError(msg)
 
         capsinfo = _CapsInfo(self.conn, guest, domain)
         return capsinfo
