@@ -4,8 +4,15 @@
 # This work is licensed under the GNU GPLv2 or later.
 # See the COPYING file in the top-level directory.
 
+from .char import CharSource
 from .device import Device
-from ..xmlbuilder import XMLProperty
+from ..xmlbuilder import XMLBuilder, XMLChildProperty, XMLProperty
+
+
+class _Certificate(XMLBuilder):
+    XML_NAME = "certificate"
+
+    value = XMLProperty("./.")
 
 
 class DeviceSmartcard(Device):
@@ -14,6 +21,10 @@ class DeviceSmartcard(Device):
 
     mode = XMLProperty("./@mode")
     type = XMLProperty("./@type")
+    source = XMLChildProperty(CharSource, is_single=True)
+
+    database = XMLProperty("./database")
+    certificates = XMLChildProperty(_Certificate)
 
 
     ##################
