@@ -13,6 +13,7 @@ from . import generatename
 from . import progress
 from .logger import log
 from .xmlbuilder import XMLBuilder, XMLChildProperty, XMLProperty
+from . import xmlutil
 
 
 _DEFAULT_DEV_TARGET = "/dev"
@@ -48,10 +49,8 @@ class _StorageObject(XMLBuilder):
 
 
 def _preferred_default_pool_path(conn):
-    path = "/var/lib/libvirt/images"
-    if conn.is_unprivileged():
-        path = os.path.expanduser("~/.local/share/libvirt/images")
-    return path
+    root = conn.get_libvirt_data_root_dir()
+    return os.path.join(root, "images")
 
 
 def _lookup_poolxml_by_path(conn, path):
@@ -84,7 +83,6 @@ class StoragePool(_StorageObject):
     TYPE_MPATH   = "mpath"
     TYPE_GLUSTER = "gluster"
     TYPE_RBD     = "rbd"
-    TYPE_SHEEPDOG = "sheepdog"
     TYPE_ZFS     = "zfs"
 
     @staticmethod
@@ -313,7 +311,7 @@ class StoragePool(_StorageObject):
 
     def supports_source_name(self):
         return self.type in [self.TYPE_LOGICAL, self.TYPE_GLUSTER,
-            self.TYPE_RBD, self.TYPE_SHEEPDOG, self.TYPE_ZFS]
+            self.TYPE_RBD, self.TYPE_ZFS]
 
 
     def supports_source_path(self):
@@ -325,7 +323,7 @@ class StoragePool(_StorageObject):
     def supports_hosts(self):
         return self.type in [
                 self.TYPE_NETFS, self.TYPE_ISCSI, self.TYPE_GLUSTER,
-                self.TYPE_RBD, self.TYPE_SHEEPDOG]
+                self.TYPE_RBD]
 
     def supports_format(self):
         return self.type in [self.TYPE_FS, self.TYPE_NETFS, self.TYPE_DISK]
@@ -342,8 +340,7 @@ class StoragePool(_StorageObject):
             return StorageVolume.TYPE_BLOCK
         if (self.type == StoragePool.TYPE_GLUSTER or
             self.type == StoragePool.TYPE_RBD or
-            self.type == StoragePool.TYPE_ISCSI or
-            self.type == StoragePool.TYPE_SHEEPDOG):
+            self.type == StoragePool.TYPE_ISCSI):
             return StorageVolume.TYPE_NETWORK
         return StorageVolume.TYPE_FILE
 
@@ -380,6 +377,10 @@ class StoragePool(_StorageObject):
         log.debug("Creating storage pool '%s' with xml:\n%s",
                       self.name, xml)
 
+        if (xmlutil.in_testsuite() and
+            "virtinst-testsuite-fail-pool-install" in xml):
+            raise RuntimeError("StoragePool.install testsuite mocked failure")
+
         meter = progress.ensure_meter(meter)
 
         try:
@@ -478,9 +479,10 @@ class StorageVolume(_StorageObject):
                     break
 
             for disk in collideguest.devices.disk:
-                if (pooltarget and disk.path and
-                    os.path.dirname(disk.path) == pooltarget):
-                    collidelist.append(os.path.basename(disk.path))
+                checkpath = disk.get_source_path()
+                if (pooltarget and checkpath and
+                    os.path.dirname(checkpath) == pooltarget):
+                    collidelist.append(os.path.basename(checkpath))
 
         def cb(tryname):
             if tryname in collidelist:
@@ -531,6 +533,7 @@ class StorageVolume(_StorageObject):
         self.format = parsevol.format
         self.capacity = parsevol.capacity
         self.allocation = parsevol.allocation
+        self.permissions.mode = parsevol.permissions.mode
         if not self._pool:
             self.pool = self._input_vol.storagePoolLookupByVolume()
 
@@ -594,7 +597,8 @@ class StorageVolume(_StorageObject):
         log.debug("Attempting to detect format for backing_store=%s",
                 self.backing_store)
         from . import diskbackend
-        vol, pool = diskbackend.manage_path(self.conn, self.backing_store)
+        path, vol, pool = diskbackend.manage_path(self.conn, self.backing_store)
+        dummy = path
 
         if not vol:  # pragma: no cover
             log.debug("Didn't find any volume for backing_store")
@@ -671,20 +675,23 @@ class StorageVolume(_StorageObject):
                 cloneflags |= libvirt.VIR_STORAGE_VOL_CREATE_PREALLOC_METADATA
 
         if self.reflink:
-            cloneflags |= getattr(libvirt,
-                "VIR_STORAGE_VOL_CREATE_REFLINK", 1)
+            if self.format != "raw":
+                log.warning("skipping reflink for non-raw vol=%s", self.name)
+            else:
+                cloneflags |= getattr(libvirt,
+                    "VIR_STORAGE_VOL_CREATE_REFLINK", 1)
 
         event = threading.Event()
         meter = progress.ensure_meter(meter)
         t = threading.Thread(target=_progress_thread,
                              name="Checking storage allocation",
                              args=(self.name, self.pool, meter, event))
-        t.setDaemon(True)
+        t.daemon = True
 
         try:
             t.start()
-            meter.start(size=self.capacity,
-                        text=_("Allocating '%s'") % self.name)
+            msg = _("Allocating '%(filename)s'") % {"filename": self.name}
+            meter.start(msg, self.capacity)
 
             if self.conn.is_really_test():
                 # Test suite doesn't support any flags, so reset them
@@ -697,7 +704,8 @@ class StorageVolume(_StorageObject):
                 log.debug("Using vol create flags=%s", createflags)
                 vol = self.pool.createXML(xml, createflags)
 
-            meter.end(self.capacity)
+            meter.update(self.capacity)
+            meter.end()
             log.debug("Storage volume '%s' install complete.", self.name)
             return vol
         except Exception as e:
